import webbrowser

import bpy

from .utils.cache import Preferences
from .icons import icon_id
from .ui import *
from .utils.version import get_this_version_name
from .utils.ui import (
    draw_paragraph,
    draw_discord_invite,
    draw_send_email,
    text_box,
    
)
from . import addon_id, bl_info
prefs = bpy.context.preferences.addons[__package__].preferences

def update_merge_panel(self, context):
    Preferences.set("merge_panel", self.merge_panel)
    update_panels()

# class AddonPreferences(bpy.types.AddonPreferences):
#     bl_idname = __package__

#     # common settings
#     update_check: bpy.props.BoolProperty(
#         name="Check for updates on startup",
#         description="Automatically check for updates every time you start Blender",
#         default=True,
#     )
#     merge_panel: bpy.props.BoolProperty(
#         name="Merge Panel",
#         description=f"Merge the {bl_info['name']} tab into the \"Rubens Addons\" tab for a cleaner Sidebar\nThis is useful when you have multiple addons from me installed and don't want to have a tab for each one",
#         default=False,
#         update=update_merge_panel,
#     )
#     menu: bpy.props.EnumProperty(
#         name="Menu",
#         description="",
#         items=[
#             ("General", "General", "", "PREFERENCES", 0),
#             ("Display", "Display", "", "MENU_PANEL", 1),
#             ("Versions", "Versions", "", "FILE_REFRESH", 3),
#         ],
#     )

#     cat_name: bpy.props.StringProperty(
#             name="Tab Category",
#             description="Choose a name for the category of the panel",
#             default=bl_info["name"],
#             # update=update_panel
#             )


class Email(bpy.types.Operator):
    bl_idname = f"{addon_id}.email"
    bl_label = "Send an Email"
    bl_options = {"INTERNAL"}
    bl_description = "Send an Email to the developer"

    include_addon_info: bpy.props.BoolProperty(default=True)
    subject: bpy.props.StringProperty()
    body: bpy.props.StringProperty()

    def execute(self, context):
        mail_to = "mailto:support@rubenmesserschmidt.com?subject="
        if self.include_addon_info:
            mail_to += f"%5B{self.to_mailto(bl_info['name'])}%5D%20"
        if self.subject:
            mail_to += f"{self.to_mailto(self.subject)}"
        mail_to += "&body="
        if self.include_addon_info:
            addon_version = ".".join(str(i) for i in get_this_version_name())
            blender_version = (
                f"{bpy.app.version[0]}.{bpy.app.version[1]}.{bpy.app.version[2]}"
            )
            os = bpy.app.build_platform.decode("utf-8")
            mail_to += (
                f"Addon%20Version%3A%20{addon_version}%0A"
                f"Blender%20Version%3A%20{blender_version}%0A"
                f"OS%3A%20{os}%0A%0A"
            )
        mail_to += self.to_mailto(self.body)

        webbrowser.open(mail_to)
        return {"FINISHED"}

    def to_mailto(self, text: str):
        return text.replace(" ", "%20").replace("\n", "%0A")
