import bpy
from bpy.types import PropertyGroup
from bpy.props import BoolProperty, FloatProperty, EnumProperty


class INSTANTCLEAN_PG_TopologySettings(PropertyGroup):
    type: EnumProperty(
        name="Type",
        items=[
            ("TRIS", "三角化", "", "MOD_TRIANGULATE", 0),
            ("QUADS", "四边面", "", "MOD_OPACITY", 1),
        ],
        default="TRIS",
        description="Type of topology which the mesh will be converted to",
    )

    # Methods
    quad_method: EnumProperty(
        name="四边面方法",
        items=[
            ("BEAUTY", "Beauty", "", "", 0),
            ("FIXED", "Fixed", "", "", 1),
            ("FIXED_ALTERNATE", "Fixed Alternate", "", "", 2),
            ("SHORTEST_DIAGONAL", "Shortest Diagonal", "", "", 3),
            ("LONGEST_DIAGONAL", "最长对角线", "", "", 4),
        ],
        description="Method which will be used for the quad to triangle computation",
    )

    quad_max_face_ang: FloatProperty(
        name="Max Face Angle",
        subtype="ANGLE",
        default=0.698123,
        description="Dissolve only edges with lower angles to create quads",
    )

    quad_max_shape_ang: FloatProperty(
        name="Max Shape Angle",
        subtype="ANGLE",
        default=0.698123,
        description="Only try to convert to a quad if the overall shape change is lower than this angle",
    )

    ngon_method: EnumProperty(
        name="NGon Method",
        items=[
            ("BEAUTY", "Beauty", "", "", 0),
            ("CLIP", "Clip", "", "", 1),
        ],
        description="Method which will be used for the ngon to triangle computation",
    )

    # Compare
    compare_sharp: BoolProperty(
        name="Sharp", default=True, description="Excludes edges marked as sharp from the convertion"
    )

    compare_seam: BoolProperty(
        name="Seam", default=True, description="Excludes edges marked as seam from the convertion"
    )

    compare_vcol: BoolProperty(
        name="顶点颜色", default=True, description="转换后保持顶点颜色不变"
    )

    compare_uv: BoolProperty(name="UV", default=True, description="Leaves uv's unchanged after convertion")

    compare_material: BoolProperty(
        name="Material",
        default=True,
        description="Excludes edges which delimit two different assigned materials from the convertion",
    )
