import json
from pathlib import Path

from .resources import get_datafiles_path


class Cache:
    filename = "cache.json"
    base_path = Path(__file__).parent.parent

    @classmethod
    def get_path(cls):
        return cls.base_path / cls.filename

    @classmethod
    def load(cls) -> dict:
        path = cls.get_path()
        if not path.exists():
            path.touch()
            path.write_text("{}")

        return json.loads(path.read_text())

    @classmethod
    def get(cls, keys):
        if isinstance(keys, str):
            keys = [keys]

        cache = cls.load()
        for key in keys:
            key = str(key)
            if key not in cache:
                return None
            cache = cache[key]
        return cache

    @classmethod
    def set(cls, keys, value):
        if isinstance(keys, str):
            keys = [keys]

        cache = cls.load()
        d = cache
        for key in keys[:-1]:
            key = str(key)
            if key not in d:
                d[key] = {}
            d = d[key]

        d[keys[-1]] = value
        cls.get_path().write_text(json.dumps(cache))

    @classmethod
    def update(cls, data):
        cls.get_path().write_text(json.dumps(data))


class Preferences(Cache):
    filename = "preferences.json"
    base_path = get_datafiles_path()
