import bpy
import bmesh


def validate_bmeshes(bmeshes):
    for bm in bmeshes:
        if not bm.is_valid:
            bmeshes.remove(bm)
            continue


def select_geo(geometry):
    for geo in geometry:
        geo.select = True


def deselect_geo(geometry):
    for geo in geometry:
        geo.select = False


def deselect_all_geo(bm):
    for type in ("verts", "edges", "faces"):
        for geo in getattr(bm, type):
            geo.select = False


def in_mode(mode: str):
    return bpy.context.mode == mode


def update_mesh(obj):
    bmesh.update_edit_mesh(obj.data)


def get_objects():
    if not bpy.context.selected_objects and not bpy.context.objects_in_mode:
        return []
    if bpy.context.mode == "OBJECT":
        return bpy.context.selected_objects
    else:
        return bpy.context.objects_in_mode


def get_bmeshs(objects):
    bmeshs = []
    for obj in objects:
        bmeshs.append(bmesh.from_edit_mesh(obj.data))

    return bmeshs


def update_bmeshs(objects):
    for obj in objects:
        bmesh.update_edit_mesh(obj.data)


def visible_verts(bm):
    return [v for v in bm.verts if not v.hide]


def visible_edges(bm):
    return [e for e in bm.edges if not e.hide]


def visible_faces(bm):
    return [f for f in bm.faces if not f.hide]


def visible_loops(bm):
    loops = []
    for f in bm.faces:
        for loop in f.loops:
            if not loop.vert.hide:
                loops.append(loop)

    return loops


def get_triangle_count(faces):
    count = 0
    for f in faces:
        count += len(f.loops) - 2

    return count
