import bpy

from .common_funcs import validate_bmeshes

from .common_funcs import validate_bmeshes


def normals(context, objects, bmeshes):
    validate_bmeshes(bmeshes)
    stats = {
        "Flipped": False,
        "Sharp Edges": False,
        "Weighted Normals": False,
    }

    pg = bpy.context.scene.instantclean_normals
    if pg.recalculate:
        stats["Flipped"] = 0
        face_normals = []
        for bm in bmeshes:
            if not bm.is_valid:
                continue
            face_normals.append([f.normal.copy() for f in bm.faces])

        bpy.ops.mesh.select_all(action="SELECT")
        bpy.ops.mesh.normals_make_consistent(
            inside=pg.recalculate_orientation == "INSIDE"
        )

        for bm, fns in zip(bmeshes, face_normals):
            if not bm.is_valid:
                continue
            stats["Flipped"] += sum(1 for f, n in zip(bm.faces, fns) if f.normal != n)

    if pg.auto_smooth:
        bpy.ops.mesh.select_all(action="SELECT")
        bpy.ops.mesh.faces_shade_smooth()
        modifier_name = "(Instant Clean) Smooth by Angle"
        for obj in objects:
            obj: bpy.types.Object
            if bpy.app.version < (4, 1, 0):
                obj.data.use_auto_smooth = True
                obj.data.auto_smooth_angle = pg.auto_smooth_ang
                continue

            if matching_modifiers := [
                m
                for m in obj.modifiers.values()
                if m.type == "NODES"
                and m.node_group.name == "Smooth by Angle"
                and m.name == modifier_name
            ]:
                mod = matching_modifiers[-1]
            else:
                mod = obj.modifiers.new(name=modifier_name, type="NODES")
                try:
                    sba_node_group = bpy.data.node_groups["Smooth by Angle"]
                except Exception:
                    bpy.ops.object.modifier_add_node_group(
                        asset_library_type="ESSENTIALS",
                        asset_library_identifier="",
                        relative_asset_identifier="geometry_nodes\\smooth_by_angle.blend\\NodeTree\\Smooth by Angle",
                    )
                    bpy.ops.object.modifier_remove(modifier="Smooth by Angle")

                mod.node_group = bpy.data.node_groups["Smooth by Angle"]

            mod["Input_1"] = pg.auto_smooth_ang

    if pg.weighted_normals:
        stats["Weighted Normals"] = 0
        modifier_name = "(Instant Clean) Weighted Normals"
        for obj in objects:
            try:
                for m in obj.modifiers.values():
                    if m.type == "WEIGHTED_NORMAL":
                        raise Exception

            except Exception:
                continue

            mod = obj.modifiers.new(name=modifier_name, type="WEIGHTED_NORMAL")
            mod.keep_sharp = True
            stats["Weighted Normals"] += 1

    if pg.clear_custom_split_normals:
        bpy.ops.mesh.select_all(action="SELECT")
        for obj in objects:
            bpy.context.view_layer.objects.active = obj
            bpy.ops.mesh.customdata_custom_splitnormals_clear()

    if pg.clear_sharp_edges:
        stats["Sharp Edges"] = 0
        for bm in bmeshes:
            if not bm.is_valid:
                continue

            for e in bm.edges:
                if e.smooth:
                    continue
                e.smooth = True
                stats["Sharp Edges"] += 1

    return stats
